"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getRef = getRef;
exports.parseRef = parseRef;
exports.fillRefs = fillRefs;
exports.getCurrentPath = exports.getRelatedForms = exports.getResponseSchema = exports.getRequestSchema = exports.parseSchema = exports.default = void 0;

var _openapiTypes = require("openapi-types");

var _objectPath = _interopRequireDefault(require("object-path"));

var _objectHash = _interopRequireDefault(require("object-hash"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var httpMethods = Object.values(_openapiTypes.OpenAPIV3.HttpMethods); // Structure to group path routes by Object tag

// convert all properties  type = integer to type = number
// to display <input type={number}... properly>
var integerToNumber = function integerToNumber(item) {
  if (Array.isArray(item)) {
    // return item.map(i => fillThing(i));
    return item;
  } else if (_typeof(item) === "object") {
    var keys = Object.keys(item);
    return keys.reduce(function (fields, k) {
      var value = integerToNumber(item[k]);

      if (k === "properties") {
        var properties = Object.keys(value).reduce(function (props, p) {
          return _objectSpread(_objectSpread({}, props), {}, _defineProperty({}, p, _objectSpread(_objectSpread({}, value[p]), {}, {
            title: value[p].title || p,
            type: value[p].type !== "integer" ? value[p].type : "number"
          })));
        }, {});
        return _objectSpread(_objectSpread({}, fields), {}, _defineProperty({}, k, properties));
      } else {
        return _objectSpread(_objectSpread({}, fields), {}, _defineProperty({}, k, value));
      }
    }, {});
  } else {
    return item;
  }
}; // Get all properties without http methods


var getPathProperties = function getPathProperties(path) {
  var keys = Object.keys(path).filter(function (p) {
    return !httpMethods.includes(p);
  });
  return keys.reduce(function (props, k) {
    return _objectSpread(_objectSpread({}, props), {}, _defineProperty({}, k, path[k]));
  }, {});
}; // Get only http methods


var getPathMethods = function getPathMethods(path, tag) {
  var keys = Object.keys(path).filter(function (p) {
    return httpMethods.includes(p) && (!tag || (path[p].tags || ["default"]).includes(tag));
  });
  return keys.reduce(function (props, k) {
    return _objectSpread(_objectSpread({}, props), {}, _defineProperty({}, k, path[k]));
  }, {});
}; // interface MethodsByTag {
//   [tag: string]: {
//     [method in OpenAPIV3.HttpMethods]: OpenAPIV3.OperationObject<any>;
//   };
// }


var groupPathByTags = function groupPathByTags(pathName, path, tagGroups) {
  var props = getPathProperties(path);
  var methods = getPathMethods(path);
  var tags = Array.from(new Set(Object.values(methods).reduce(function (tagList, method) {
    return [].concat(_toConsumableArray(tagList), _toConsumableArray(method.tags || ["default"]));
  }, [])));
  return tags.reduce(function (methodsByTag, tag) {
    var tagGroup = methodsByTag[tag] || {};
    var content = tagGroup[pathName] || {};
    return _objectSpread(_objectSpread({}, methodsByTag), {}, _defineProperty({}, tag, _objectSpread(_objectSpread({}, tagGroup), {}, _defineProperty({}, pathName, _objectSpread(_objectSpread(_objectSpread({}, props), content), getPathMethods(path, tag))))));
  }, tagGroups); // const methodsByTag = Object.keys(methods).reduce( ())
};

var groupByTag = function groupByTag(api) {
  return Object.keys(api.paths).reduce(function (tagGroups, pathName) {
    return groupPathByTags(pathName, api.paths[pathName], tagGroups);
  }, {});
};

var parseSchema = function parseSchema(api) {
  return groupByTag(integerToNumber(fillRefs(api, api)));
};

exports.parseSchema = parseSchema;
var _default = parseSchema;
exports.default = _default;

function getRef(API, refPath) {
  var path = refPath.replace(/^#\//, "").split("/");
  return _objectPath.default.get(API, path.join("."));
}

function parseRef(API, item) {
  if (item["$ref"]) {
    return getRef(API, item["$ref"]);
  } else {
    return item;
  }
}

function fillRefs(API, dict) {
  if (_typeof(dict) === "object" && !Array.isArray(dict)) {
    return Object.keys(dict).reduce(function (newObj, key) {
      var content = key === "$ref" ? fillRefs(API, getRef(API, dict[key])) : _defineProperty({}, key, fillRefs(API, dict[key]));
      return _objectSpread(_objectSpread({}, newObj), content);
    }, {});
  } else if (Array.isArray(dict)) {
    return dict.map(function (key) {
      return fillRefs(API, key);
    });
  } else {
    return dict;
  }
}

var getRequestSchema = function getRequestSchema(path, method) {
  var pathObject = path[method];
  if (!pathObject || !pathObject.requestBody) return undefined; // const requestBody = fillRefs<OpenAPIV3.RequestBodyObject>(
  //   API,

  var requestBody = pathObject.requestBody; // );

  if (!requestBody.content["application/json"].schema) return undefined;
  return requestBody.content["application/json"].schema; // try {
  //   const schema = fillRefs<OpenAPIV3.SchemaObject>(
  //     API,
  //     requestBody.content["application/json"].schema
  //   );
  //   return schema;
  // } catch (e) {
  //   throw new Error(
  //     `Cannot find requestBody.content["application/json"].schema `
  //   );
  // }
};

exports.getRequestSchema = getRequestSchema;

var getResponseSchema = function getResponseSchema(path, method, code) {
  var pathObject = path[method];
  if (!pathObject || !pathObject.responses || !pathObject.responses[String(code)]) return undefined;
  var response = pathObject.responses[String(code)];
  if (!response.content || !response.content["application/json"].schema) return undefined;
  return response.content["application/json"].schema;
};

exports.getResponseSchema = getResponseSchema;

var getRelatedRequests = function getRelatedRequests(API, responseHash) {
  if (!responseHash) {
    return {};
  }

  var paths = Object.keys(API.paths);
  return paths.reduce(function (map, pathName) {
    var methods = Object.keys(API.paths[pathName]);
    var writers = [_openapiTypes.OpenAPIV3.HttpMethods.POST, _openapiTypes.OpenAPIV3.HttpMethods.PUT, _openapiTypes.OpenAPIV3.HttpMethods.PATCH];
    var writeMethods = methods.filter(function (method) {
      return writers.includes(method);
    }).filter(function (method) {
      var requestBody = API.paths[pathName][method].requestBody;
      var content = requestBody && requestBody.content;
      var requestSchema = content && content["application/json"] && content["application/json"].schema;
      var requestHash = requestSchema && (0, _objectHash.default)(requestSchema);
      return !!requestHash && requestHash === responseHash;
    });
    var related = writeMethods.length ? _defineProperty({}, pathName, writeMethods) : {};
    return _objectSpread(_objectSpread({}, map), related);
  }, {});
};

var getRelatedForms = function getRelatedForms(API) {
  // const readMethods = ['GET']
  // const writeMethods = ['POST', 'PUT', 'PATCH'];
  var paths = Object.keys(API.paths);
  return paths.reduce(function (map, pathName) {
    var methods = Object.keys(API.paths[pathName]);
    var readMethod = methods.find(function (method) {
      return method === _openapiTypes.OpenAPIV3.HttpMethods.GET;
    });

    if (readMethod) {
      var responses = API.paths[pathName][readMethod].responses;
      var content = responses["200"] && responses["200"].content;
      var responseSchema = content && content["application/json"] && content["application/json"].schema;
      var responseHash = responseSchema && (0, _objectHash.default)(responseSchema);
      var related = getRelatedRequests(API, responseHash);

      if (Object.keys(related).length) {
        return _objectSpread(_objectSpread({}, map), {}, _defineProperty({}, pathName, related));
      }

      return map;
    } // const data = !!readMethod ? {
    //   [readMethod]:
    // }: {};


    return map;
  }, {});
};

exports.getRelatedForms = getRelatedForms;

var getCurrentPath = function getCurrentPath(api, pathname) {
  var paths = Object.keys(api.paths);
  var repaths = paths.map(function (path) {
    return new RegExp("^" + path.replaceAll("/", "\\/").replace(/\{[^}]+\}/, "([^\\/]+)") + "$");
  });
  var match;

  for (var i in repaths) {
    match = pathname.match(repaths[i]);

    if (match) {
      return {
        match: match,
        path: paths[i]
      };
    }
  }

  return match;
};

exports.getCurrentPath = getCurrentPath;